#include "particleSystem.h"

///konstruktor
ParticleSystemBod::ParticleSystemBod(int x, int y, Barva &barva1, Barva &barva2,
															 int mnozstvi, int dolniMez, int horniMez) : parametry()
{
	parametry.x = x;
	parametry.y = y;
	pocet = mnozstvi;
	parametry.pocatecni = barva1;
	parametry.koncova = barva2;
	uhelHorni = horniMez - dolniMez + 1;
	uhelDolni = 360 - horniMez;

	castice = new Castice*[pocet];
	for (int i = 0; i < pocet; i++)
		vytvorCastici(i);
}

///destruktor
ParticleSystemBod::~ParticleSystemBod()
{
	for (int i = 0; i < pocet; i++)
		delete castice[i];
	delete []castice;
}

///zmena poctu castic generovnych Particle systemem
void ParticleSystemBod::zmenPocetCastic(int mnozstvi)
{
	Castice **pomocna = castice;
	castice = new Castice*[mnozstvi];

	for (int i = 0; i < mnozstvi; i++) {
		if (i < pocet) {
			castice[i] = pomocna[i];
			pomocna[i] = NULL;
		}
		else
			vytvorCastici(i);
	}
	for (int i = mnozstvi; i < pocet; i++)
		delete pomocna[i];
	delete []pomocna;
	pocet = mnozstvi;
}

///vytvari novou castici pro generovani
void ParticleSystemBod::vytvorCastici(int i)
{
	parametry.uhel = rand() % uhelHorni + uhelDolni;
	castice[i] = new Castice(parametry);
}

///opravuje polohu castice v prubehu vykreslovani
void ParticleSystemBod::opravCastici(int i)
{
	castice[i]->posunSe();
	if (!castice[i]->jeZiva()) {
		delete castice[i];
		parametry.uhel = rand() % uhelHorni + uhelDolni;
		castice[i] = new Castice(parametry);
	}
}

///znovu vytvori castice ze zadanych parametru
void ParticleSystemBod::reset()
{
	for (int i = 0; i < pocet; i++) {
		delete castice[i];
		vytvorCastici(i);
	}
}

///zobrazovani generovanych castic ve vyseci definovane dvema uhly
void ParticleSystemBod::zobraz()
{
	Barva pomocna;
	for (int i = 0; i < pocet; i++) {
		pomocna = castice[i]->vratBarvu();
		al_put_pixel(castice[i]->vratX(), castice[i]->vratY(), al_map_rgb(pomocna.r, pomocna.g, pomocna.b));
		opravCastici(i);
	}
}

///konstruktor generovani castic v pruhu, ktery bude ve smeru uhlu
ParticleSystemPas::ParticleSystemPas(int x, int y, Barva &b1, Barva &b2,
																		int pocet, int delka, int uhel) : ParticleSystemBod(x, y, b1, b2, pocet)
{
	if (uhel != -90 && uhel != 180 && uhel != 90 && uhel != 0)
		uhel = -90;

	sirka = delka;
	parametry.uhel = uhel;
	reset();
}

///vytvori novou castici na indexu zadanem v parametru
void ParticleSystemPas::vytvorCastici(int i)
{
	int pomX = parametry.x, pomY = parametry.y;

	if (parametry.uhel == 90 || parametry.uhel == -90)
		parametry.x = pomX + rand() % sirka + 1;
	else
		parametry.y = pomY + rand() % sirka + 1;
	castice[i] = new Castice(parametry);

	parametry.x = pomX;
	parametry.y = pomY;
}

///zmeni smer generovanych castic
void ParticleSystemPas::zmenSmer(int uhel)
{
	if (uhel == -90 || uhel == 180 || uhel == 90 || uhel == 0)
		parametry.uhel = uhel;
}

///opravuje polohu castice v prubehu vykreslovani
void ParticleSystemPas::opravCastici(int i)
{
	castice[i]->posunSe();
	if (!castice[i]->jeZiva()) {
		delete castice[i];
		vytvorCastici(i);
	}
}
